/*
 *  linux/arch/ppc/kernel/setup.c
 *
 *  TCD version
 *    Copyright (C) 1998, 1999 TiVo Inc. All Rights Reserved.
 *
 *  PowerPC version
 *    Copyright (C) 1995-1996 Gary Thomas (gdt@linuxppc.org)
 *
 *  Adapted for Power Macintosh by Paul Mackerras
 *    Copyright (C) 1996 Paul Mackerras (paulus@cs.anu.edu.au)
 *
 *  Derived from "arch/alpha/kernel/setup.c"
 *    Copyright (C) 1995 Linus Torvalds
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version
 *  2 of the License, or (at your option) any later version.
 *
 */

/*
 * bootup setup stuff..
 */

#include <linux/config.h>
#include <linux/errno.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/stddef.h>
#include <linux/unistd.h>
#include <linux/ptrace.h>
#include <linux/malloc.h>
#include <linux/user.h>
#include <linux/a.out.h>
#include <linux/tty.h>
#include <linux/string.h>
#include <linux/delay.h>
#include <linux/ioport.h>
#include <linux/major.h>
#include <linux/ide.h>
#include <linux/init.h>
#include <linux/console.h>
#include <linux/tivoconfig.h>
#include <asm/prom.h>
#include <asm/system.h>
#include <asm/pgtable.h>
#include <asm/io.h>
#include <asm/ppcops.h>

#ifdef  CONFIG_TIVO
/*
 * We need this to compile the generic (that come with the source code)
 * ide drivers.
 */
struct drive_info_struct { char dummy[32]; } drive_info;
#endif
struct ide_machdep_calls ppc_ide_md = { 0 };

extern int root_mountflags;

void do_unblank_screen(void);
void hard_reset_now(void);
void console_map_init(void);
unsigned long con_init(unsigned long);
void init_command_line(void);

#define COMMAND_LINE_SIZE 256
static char command_line[COMMAND_LINE_SIZE] = { 0, };
char saved_command_line[COMMAND_LINE_SIZE] = { 0, };

static char saved_boot_params[COMMAND_LINE_SIZE];
struct machdep_calls ppc_md;

unsigned long __init
find_end_of_memory(unsigned long end) 
{
	extern int PromMemConfig;
	return(0x80000000 + (PromMemConfig * 1024 * 1024));
}

void __init setup_arch(char **cmdline_p,
	unsigned long * memory_start_p, unsigned long * memory_end_p)
{
	extern char _end[];
	extern unsigned long loops_per_sec;
	extern int _xdebug;
	extern void _xdebug_entry(void);
	struct console *s403_virtualinit(void);
	unsigned int iocr;
	unsigned int br;

	/* HACK: check for first 5 chars == Root=.  If so, rewrite root= and
	   disable console */
	if (!strncmp(saved_boot_params, "Root=", 5)) {
		extern int s403_ready;

		saved_boot_params[0] = 'r';
		s403_ready = -1;
	}
	register_console(s403_virtualinit());
	printk("\rTiVo:-) Customer Device Starting Up!\n");

#ifdef CONFIG_XMON
	if (_xdebug & 0x1)
		_xdebug_entry();
#else
	mtspr(SPRN_DBCR, 0x40000000);	/* enter internal debug mode */
#endif
	mtdcr(SPRN_BESR, 0);		/* make sure it's unlocked */

	*memory_start_p = ((unsigned long)_end + PAGE_SIZE)&PAGE_MASK;
	*memory_end_p = find_end_of_memory(*memory_start_p);

	{
	  extern void DmaChInitAll(void);
	  DmaChInitAll();
	}
	
	/* insure external interrupts are set up properly */
	iocr = mfdcr(SPRN_IOCR);
	iocr &= ~0xffc00000;
	iocr |= 0x05400000;
	iocr |= 0x00000400;	/* ignore device timeouts for now */
	mtdcr(SPRN_IOCR, iocr);

	/* special bank register tweaks */

	br = mfdcr(SPRN_BR4);
	br &= ~0xff000000;	/* set up just like uart */
	br |= ((IOBASE_MODEM << 4) & 0xff000000);
	mtdcr(SPRN_BR3, br);

	{
		unsigned long iocr = mfspr(SPRN_IOCR);
		mtspr(SPRN_IOCR, iocr | 0x00000400);	/* insure we'll timeout */
	}

	/*
	 * Hardcoded NFS boot, for now.
	 */
	init_command_line();
	*cmdline_p = command_line;

	/* Set loops_per_sec to a half-way reasonable value,
	   for use until calibrate_delay gets called. */
        loops_per_sec = (27 * 2 * 1000000);
}

char *bootpath;
char bootdevice[256];
void *boot_host;
int boot_target;
int boot_part;
kdev_t boot_dev;

int
get_cpuinfo(char *buffer)
{
	int l;
	unsigned int pvr = _get_PVR();

	l = sprintf(buffer, "processor\t: 0\ncpu\t\t: IBM 403GCX\nclock\t\t: 54MHz\n");
	l += sprintf(buffer+l, "revision\t: %d.%d\n",
		     (pvr & 0xff00) >> 8, pvr & 0xff);
	l += sprintf(buffer+l, "bogomips\t: %lu.%02lu\n",
		     (loops_per_sec + 2500) / 500000,
		     (loops_per_sec + 2500) / 5000 % 100);
	l += sprintf(buffer+l, "machine\t\t: Teleworld Customer Device\n");
	return l;
}

void
abort()
{
#ifdef CONFIG_XMON
	xmon(0);
#endif
	printk("Abort with no net; entering an infinite loop\n");
	for (;;) {}
}

void do_unblank_screen() {}

void __init console_map_init() {}

void __init ppc_init(void) {}

unsigned long __init con_init(unsigned long memstart) {
	/* eventally register on-board serial for bring-up */
	return memstart;
}

void machine_restart(char *cmd)
{
	/* cause a full system reset */
	mtspr(SPRN_DBCR, 0x30000000);
}
  
void machine_halt(void)
{
	printk("Halt reqeuested, restart system to continue.\n");
	for (;;) {}
}

void machine_power_off(void)
{
	machine_halt();
}

int proc_dol2crvec(void *table, int write, void *filp, void *buffer, size_t *lenp)
{
    panic("Called proc_dol2crvec; don't do that on the 4xx!\n");
}

void note_bootable_part(kdev_t dev, int part)
{
}
 
/* NOTE: this awful hack is valid only for the PX prototype boards. */
/* The ethernet board has a 3 bit id register at the following address. */
/* The 8 bits read contain the id in bits 1-3: XXXX IIIX */
#define PROMADDR 0xfff00000

#define MAXDEVICES 6

struct confdata {
        long    nvramid[2];             /* Validity check field            */
        int     conf_size;              /* Size of this structure          */
        int     numdev;                 /* Number of entries               */
        int     nomenu;                 /* Wait for input                  */
        int     gotodbg;                /* Exit to debugger                */
        int     numboot;                /* Number of boot devices          */
        int     numtest;                /* Number of device tests          */
	char	mac_address[6];
	char	hostname[32];
        struct {
                unsigned long src;      /* Source IP address               */
                unsigned long dst;      /* Dest. IP address                */
                char    present;        /* device present                  */
                char    noboot;         /* Don't boot from device          */
                char    bootorder;      /* boot order                      */
                char    notest;         /* Don't test device               */
                int     baud;           /* Baud rate (for s1 download only)*/
        } ipaddr[MAXDEVICES];
	char	bootparam[128];
	int	memconfig;
	int	videoconfig;
};

char hw_enet_addr[6];
int TiVoBrdType = 0;

/* Find configured mem size.  If there is a boot parameter "PHYSMEM=NN", use
   that number and ignore the configuration parameter from the PROM.  */
int __init get_memconfig(void)
{
	struct confdata *conf;
	int memsize = 0;
	char *p;
	extern unsigned int ms_base_addr;
	TivoConfigValue configValue = kTivoConfigBoardIDClassic;

	p = saved_boot_params;

	/* Look for " PHYSMEM=NN" */
	while (*p) {
		if (*p++ == ' ')
			if (!strncmp(p, "PHYSMEM=", 8)) {
				p += 8;
				if (*p >= '0' && *p <= '9')
					memsize = (*p++) - '0';
				if (*p >= '0' && *p <= '9') {
					memsize *= 10;
					memsize += (*p) - '0';
				}
				printk("Memory configured for %dMB\n", memsize);
				return(memsize);
			}
	}
	conf = (struct confdata *)PROMADDR;

	if (ms_base_addr) printk("ms_base_addr already initialized.\n");
	if (conf->videoconfig == 5) {
	        configValue = kTivoConfigBoardIDCombo;
		ms_base_addr = NEW_MS_BASE_ADDR;
		printk("New MediaSwitch address map\n");
	} else {
		ms_base_addr = OLD_MS_BASE_ADDR;
		printk("Old MediaSwitch address map\n");
	}
	if (conf->memconfig < 0)
	   memsize = -conf->memconfig;
	else
	   memsize = conf->memconfig;
        if (!memsize)
	  memsize = 8;

	InitTivoConfig( configValue );
	SetTivoConfig( kTivoConfigMemSize, memsize );

	printk("Memory configured for %dMB\n", memsize);
	return(memsize);
}


void __init save_boot_params(char *params)
{
	strncpy(saved_boot_params, params, 128);
}

void __init init_command_line()
{
	int ip;
	struct confdata *conf;
	char *param;

	conf = (struct confdata *)PROMADDR;

	if (saved_boot_params[0])
		param = saved_boot_params;
	else
		param = conf->bootparam;
	printk("Boot Parameters=%s\n", param);
	memcpy(hw_enet_addr, conf->mac_address, 6);
	printk("Initial MAC address=%x:%x:%x:%x:%x:%x\n", 
		hw_enet_addr[0]&0xff, hw_enet_addr[1]&0xff, 
		hw_enet_addr[2]&0xff, hw_enet_addr[3]&0xff,
		hw_enet_addr[4]&0xff, hw_enet_addr[5]&0xff);
	/* Specify nfs=44:/home/sstone/TiVoRoot */
	/* Expands into nfsaddrs=TCDIPADDR:192.168.1.xx:192.168.1.240:255.255.255.0:hostname /home/sstone/TiVoRoot" */
	if (!strncmp("nfs=", param, 4)) {
		char *p = param + 4;
		char *hostip;
		ip = conf->ipaddr[1].src;

		hostip = p;
		while (*p && (*p != ':')) p++;
		if (*p) *p++ = 0;
		sprintf(command_line, 
			"nfsaddrs=%d.%d.%d.%d:192.168.1.%s:192.168.1.240:255.255.255.0"
			":%s nfsroot=%s",
			(ip>>24) & 0xff, (ip>>16) & 0xff, (ip>>8) & 0xff,
			ip & 0xff, hostip, conf->hostname, p);
		printk("Command line: %s\n", command_line);
		return;
	}
	if (!strncmp("xnfs=", param, 5)) {
		char *p = param + 5;
		char *hostip;
		ip = conf->ipaddr[1].src;

		hostip = p;
		while (*p && (*p != ':')) p++;
		if (*p) *p++ = 0;
		sprintf(command_line, 
			"nfsaddrs=%d.%d.%d.%d:%s:192.168.1.240:255.255.255.0"
			":%s nfsroot=%s",
			(ip>>24) & 0xff, (ip>>16) & 0xff, (ip>>8) & 0xff,
			ip & 0xff, hostip, conf->hostname, p);
		printk("Command line: %s\n", command_line);
		return;
	}
	if (!strcmp("nfs", param)) {
		ip = conf->ipaddr[1].src;
		sprintf(command_line, 
			"nfsaddrs=%d.%d.%d.%d:192.168.1.44:192.168.1.240:255.255.255.0"
			":%s nfsroot=/usr/teleworld/tcdroot/", 
			(ip>>24) & 0xff,
			(ip>>16) & 0xff,
			(ip>>8) & 0xff,
			ip & 0xff, conf->hostname);
		printk("Command line: %s\n", command_line);
		return;
	} 
	if (!strcmp("nfs1", param)) {
		ip = conf->ipaddr[1].src;
		sprintf(command_line, 
			"nfsaddrs=%d.%d.%d.%d:192.168.1.44:192.168.1.240:255.255.255.0"
			":%s nfsroot=/home/sstone/tcdroot/", 
			(ip>>24) & 0xff,
			(ip>>16) & 0xff,
			(ip>>8) & 0xff,
			ip & 0xff, conf->hostname);
		printk("Command line: %s\n", command_line);
		return;
	} 
	if (!strcmp("nfs2", param)) {
		ip = conf->ipaddr[1].src;
		sprintf(command_line, 
			"nfsaddrs=%d.%d.%d.%d:192.168.1.44:192.168.2.227:255.255.255.0"
			":%s nfsroot=/usr/teleworld/tcdroot/", 
			(ip>>24) & 0xff,
			(ip>>16) & 0xff,
			(ip>>8) & 0xff,
			ip & 0xff, conf->hostname);
		printk("Command line: %s\n", command_line);
		return;
	} 
	strcpy(command_line, param);
}
