/*
 * Copyright (C) 1996 Paul Mackerras.
 */
#include <linux/string.h>
#include <asm/ppcops.h>
#include <linux/autoconf.h>

#ifdef CONFIG_XMON

extern int	s403_getchar(void);
extern void	s403_putchar(int);
extern void	s403_print(const char *);

void xmon_printk(char *, int, int);
void xmon_sprintf(char *, const char *, ...);
extern s403_ready;

int
xmon_write(void *handle, void *ptr, int nb)
{
    char *p = ptr;
    int i;

    for (i = 0; i < nb; ++i)
    	s403_putchar(*p++);
    return i;
}

int
xmon_read(void *handle, void *ptr, int nb)
{
    char *p = ptr;
    int i;

    for (i = 0; i < nb; ++i)
    	*p++ = s403_getchar();
    return i;
}

int
xmon_exit(void)
{
    xmon_printk("XMON exit, but don't know how!\n", 0, 0);
    for (;;) {}
}

void *xmon_stdin;
void *xmon_stdout;
void *xmon_stderr;

void
xmon_init(void)
{
}

int
xmon_putc(int c, void *f)
{
    char ch = c;

    if (c == '\n')
	xmon_putc('\r', f);
    return xmon_write(f, &ch, 1) == 1? c: -1;
}

int
xmon_putchar(int c)
{
    return xmon_putc(c, xmon_stdout);
}

int
xmon_fputs(char *str, void *f)
{
    int n = strlen(str);

    return xmon_write(f, str, n) == n? 0: -1;
}

int
xmon_readchar(void)
{
    char ch;

    if (s403_ready < 0)
      return -1;
    for (;;) {
	switch (xmon_read(xmon_stdin, &ch, 1)) {
	case 1:
	    return ch;
	case -1:
	    xmon_printk("read(stdin) returned -1\r\n", 0, 0);
	    return -1;
	}
    }
}

static char line[256] = { '\0' };

char *
xmon_readcmd(void) {
	char *lineptr = line;

	if (s403_ready < 0) {
	    /* No console.  Shouldn't be in xmon, just force 'c'. */
	    line[0] = 'c';
	    line[1] = '\n';
	    line[2] = 0;
	    return line;
	}
	for (;;) {
	    int c = s403_getchar();
	    if (c == -1 || c == 4)
		break;
	    if (c == '\r' || c == '\n') {
		*lineptr++ = '\n';
		if (c == '\r')
			s403_putchar('\r');
		s403_putchar('\n');
		break;
	    }
	    switch (c) {
	    case 0177:
	    case '\b':
		if (lineptr > line) {
		    s403_putchar('\b');
		    s403_putchar(' ');
		    s403_putchar('\b');
		    --lineptr;
		}
		break;
	    case 'U' & 0x1F:
		while (lineptr > line) {
		    s403_putchar('\b');
		    s403_putchar(' ');
		    s403_putchar('\b');
		    --lineptr;
		}
		break;
	    default:
		if (lineptr >= &line[sizeof(line) - 1])
		    s403_putchar('\a');
		else {
		    s403_putchar(c);
		    *lineptr++ = c;
		}
	    }
	}
	*lineptr = '\0';
	return line;
}

void
xmon_printk(char *fmt, int a1, int a2)
{
    char buf[128];

    xmon_sprintf(buf, fmt, a1, a2);
    xmon_write(xmon_stdout, buf, strlen(buf));
}
#endif CONFIG_XMON
